
#include	<Mstn\MdlApi\MdlApi.h>
#include	"FileCollector.h"
#include	"BoostFileSystem.h"
#include	<Shobjidl.h>			//	Windows Shell API
#include	"MicroStation.h"
#include	"FileFilterFunctors.h"
#include 	"MessageTracer.h"
#include	<tchar.h>				//	Windows TCHAR macro

///////////////////////////////////////////////////////////////////////
using namespace boost::filesystem;
using namespace Bentley;
using namespace DgnPlatform;
///////////////////////////////////////////////////////////////////////
//	static
std::wstring		FileCollector::GetProjectRoot		()
{
	return std::wstring (MicroStation::GetCfgVar (L"_USTN_PROJECTDATA").c_str ());
}
///////////////////////////////////////////////////////////////////////
//	static
//	See:
//	http://www.jasinskionline.com/windowsapi/ref/s/shgetfileinfo.html
//	http://bcbjournal.org/articles/vol4/0002/Getting_shell_item_information.htm
std::wstring		FileCollector::DescribeFileType		(path const&	p)
{
	::SHFILEINFO	fileInfo	= { 0 };
	::SHGetFileInfo (p.filename ().wstring ().c_str (), FILE_ATTRIBUTE_ARCHIVE, &fileInfo, sizeof (fileInfo),
		SHGFI_USEFILEATTRIBUTES | SHGFI_TYPENAME);
	std::wstring	descr	(fileInfo.szTypeName);
	return descr;
}
///////////////////////////////////////////////////////////////////////
//	static
void			FileCollector::RemoveExtension		(path&	p)
{
	//	Swap the current extension for an empty one
	path		empty;
	p.replace_extension (empty);
}
///////////////////////////////////////////////////////////////////////
size_t			FileCollector::Harvest		(WChar const*			root,
											bool					subFolders,
											bool					cadFiles,
											bool					writable)
{
	root_		= root;
	recurse_	= subFolders;
	cad_		= cadFiles;
	writable_	= writable;
	return Harvest ();
}
///////////////////////////////////////////////////////////////////////
size_t			FileCollector::Harvest		(std::wstring const&	root,
											bool					subFolders,
											bool					cadFiles,
											bool					writable)
{
	return Harvest (root.c_str (), subFolders, cadFiles, writable);
}
///////////////////////////////////////////////////////////////////////
size_t			FileCollector::Harvest		()
{
	Bentley::WString	msg;
	WChar				buffer	[_MAX_PATH + 20];
	swprintf (buffer, L"FileCollector::Harvest root '%s'", root_.c_str ());
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Debug, buffer, buffer, DgnPlatform::OutputMessageAlert::None);
	if (root_.empty ())
	{
		mdlOutput_rscsPrintf (msg, 0, MESSAGELIST_Errors, ERROR_RootPathEmpty);
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Warning, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::Dialog);
		return 0;
	}

	path 	p (root_);
	if (!exists (p))
	{
		mdlOutput_rscsPrintf (msg, 0, MESSAGELIST_Errors, ERROR_RootPathInvalid, root_.c_str ());
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Warning, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::Dialog);
		return 0;
	}

	MicroStation::BusyCursor	busy;
	if (recurse_)
	{
		HarvestRecursive (p);
	}
	else
	{
		HarvestRootOnly (p);
	}

	std::sort (files_.begin (), files_.end ());
	return files_.size ();
}
///////////////////////////////////////////////////////////////////////
bool			FileCollector::IsCadFile			(path const& p)	const
{
	CadFile		test (CadFiles);
	const bool cadFile	= test.operator ()(p);
	return cadFile;
}
///////////////////////////////////////////////////////////////////////
bool			FileCollector::InsertPath			(path const& p)
{
	bool		collected	(false);
	file_status fs = status (p);
	switch (fs.type ())
	{
	case regular_file:
		if (cad_)
			collected	= IsCadFile (p);
		else
			collected	= true;
		break;
	case symlink_file:
		break;
	case directory_file:
		break;
	default:
		break;
	}

	if (fs.permissions() & owner_write)
	{

	}
	else
	{
		if (writable_)
			collected	= false;
	}

	if (collected)
	{
		files_.push_back (p.wstring ());
		paths_.push_back (p);
	}
	return collected;
}
///////////////////////////////////////////////////////////////////////
size_t			FileCollector::HarvestRecursive		(path const& p)
{
	Bentley::WString	msg;
	mdlOutput_rscsPrintf (msg, 0, MESSAGELIST_Messages, MESSAGE_Recurse, root_.c_str ());
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Info, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::None);
	recursive_directory_iterator	begin (p);
	recursive_directory_iterator	end;
	for (; begin != end; ++begin)
	{
		InsertPath (*begin);
	}

	return files_.size ();
}
///////////////////////////////////////////////////////////////////////
size_t			FileCollector::HarvestRootOnly		(path const& p)
{
	Bentley::WString	msg;
	mdlOutput_rscsPrintf (msg, 0, MESSAGELIST_Messages, MESSAGE_RootOnly, root_.c_str ());
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Info, msg.c_str (), msg.c_str (), DgnPlatform::OutputMessageAlert::None);
	directory_iterator begin (p);
	directory_iterator end;
	for (; begin != end; ++begin)
	{
		InsertPath (*begin);

	}

	return files_.size ();
}
///////////////////////////////////////////////////////////////////////
void			FileCollector::Trace		()	const
{
	WChar					buffer	[_MAX_PATH + 20];
	UInt32					nFile	= 0;
	WideStringConstIterator	it;
	WideStringConstIterator	end	= files_.end ();
	for (it = files_.begin (); it != end; ++it, ++nFile)
	{
		swprintf (buffer, L"File [%d] '%s'", nFile, it->c_str ());
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Debug, buffer, buffer, DgnPlatform::OutputMessageAlert::None);
	}

	swprintf (buffer, L"Harvested [%d] files", files_.size ());
	mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Debug, buffer, buffer, DgnPlatform::OutputMessageAlert::None);
}
//////////////////////////////////////////////////////////////////////
tstring			FileCollector::GetFileTypeDescr		(LPCTSTR			file)	const
{
	::SHFILEINFO	fileInfo	= { 0 };
	::SHGetFileInfo (file, 0, &fileInfo, sizeof (fileInfo), SHGFI_TYPENAME);
	return tstring (fileInfo.szTypeName);
}
//////////////////////////////////////////////////////////////////////
tstring			FileCollector::GetFileTypeInfo		(LPCTSTR			filePath,
													DgnFileFormatType&	fileFormat,
													int&				dimension)	const
{
	bool				is3D				(false);
	DgnFileFormatType	dgnFormat			(MicroStation::FileFormat (filePath, &is3D));

	switch (fileFormat)
	{
	case DgnFileFormatType::V8:
	case DgnFileFormatType::V7:
	case DgnFileFormatType::DWG:
	case DgnFileFormatType::DXF:
		dimension	= is3D? 3: 2;
		fileFormat	= dgnFormat;
		break;
	default:
		dimension	= 0;
		fileFormat	= DgnFileFormatType::Invalid;
		break;
	}

	return GetFileTypeDescr (filePath);
}
//////////////////////////////////////////////////////////////////////
tstring		FileCollector::GetFileMode			(LPCWSTR			file)	const
{
	::SHFILEINFO	shFileInfo	= { 0 };
	::SHGetFileInfo (file, 0, &shFileInfo, sizeof (shFileInfo), SHGFI_ATTRIBUTES);
	tstring			mode		= _T("");
	const tstring	Comma		= _T(",");
	const tstring	Space		= _T(" ");
	if (shFileInfo.dwAttributes & SFGAO_LINK)
	{
		mode	= _T("Shortcut");
	}

	if (shFileInfo.dwAttributes & SFGAO_FOLDER)
	{
		if (mode.empty ())
			mode	= _T("Folder");
		else
			mode	+= Comma + Space + _T("Folder");
	}

	if (shFileInfo.dwAttributes & SFGAO_HIDDEN)
	{
		if (mode.empty ())
			mode	= _T("Hidden");
		else
			mode	+= Comma + Space + _T("Hidden");
	}

	if (shFileInfo.dwAttributes & SFGAO_READONLY)
	{
		if (mode.empty ())
			mode	= _T("Read-only");
		else
			mode	+= Comma + Space + _T("Read-only");
	}
	else
	{
		if (mode.empty ())
			mode	= _T("R/W");
		else
			mode	+= Comma + Space + _T("R/W");
	}

	if (shFileInfo.dwAttributes & SFGAO_REMOVABLE)
	{
		if (mode.empty ())
			mode	= _T("Removeable");
		else
			mode	+= Comma + Space + _T("Removeable");
	}

	if (shFileInfo.dwAttributes & SFGAO_SHARE)
	{
		if (mode.empty ())
			mode	= _T("Shared");
		else
			mode	+= Comma + Space + _T("Shared");
	}
	return mode;
}
///////////////////////////////////////////////////////////////////////

