#define DEBUG_VERSION 1

/*----------------------------------------------------------------------+
|									|
| Copyright , Bentley Systems, Incorporated, 			|
| All rights reserved.							|
|									|
|									|
| Limited permission is hereby granted to reproduce and modify this	|
| copyrighted material provided that the resulting code is used only in |
| conjunction with Bentley Systems products under the terms of the	|
| license agreement provided therein, and that this notice is retained	|
| in its entirety in any such reproduction or modification.		|
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|									|
|   $Workfile:   datasheet  $
|   $Revision:  $
|   	$Date:  $
|									|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   datasheet - datasheet source code. |
|                                                                       |
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   Include Files                                                       |
|                                                                       |
+----------------------------------------------------------------------*/
#include    <Mstn\MdlApi\MdlApi.h>
#include    <Mstn\cmdlist.r.h>
#include    <string.h>

#include 	"FileCollectorCmd.h"
#include 	"FileCollector.h"
#include 	"BoostFileSystem.h"
#include 	"List_Model.h"
#include 	"MessageTracer.h"
#include 	"MicroStation.h"

#include	<tchar.h>	//	Windows TCHAR macro

GlobalData  gData;

//////////////////////////////////////////////////////////////////////
void			TraceFlags		(GlobalData const&	data)
{
	CMessageTracer	msg	("Harvest flags: ");
	msg.Append ("sub-folders "); msg.Append (data.flags.subFolders); msg.Space ();
	msg.Append ("CAD files "); msg.Append (data.flags.cadFiles); msg.Space ();
	msg.Append ("Writable files "); msg.Append (data.flags.writableFiles); msg.Space ();
	msg.Output ();
}
//////////////////////////////////////////////////////////////////////
DialogItemP		GetFileListBox (MSDialogP dbP)
{
	DialogItemP	pListBox	= nullptr;
	if (nullptr == dbP)
	{
		CMessageTracer	msg ("GetFileListBox dbP is nullptr");
		msg.Warning (DgnPlatform::OutputMessagePriority::Warning);
	}
	else
	{
		pListBox	= mdlDialog_itemGetByTypeAndId (dbP, RTYPE_ListBox, LISTBOX_FileCollector, 0);
		if (nullptr == pListBox)
		{
			CMessageTracer	msg ("GetFileListBox pListBox is nullptr");
			msg.Warning (DgnPlatform::OutputMessagePriority::Warning);
		}
		ListModelP	pListModel	= mdlDialog_listBoxGetListModelP (pListBox->rawItemP);
		CMessageTracer	msg ("GetFileListBox pListModel_ is ");
			msg.Append (nullptr == pListModel? "nullptr": "valid");
		msg.Output ();
	}
	return pListBox;
}
//////////////////////////////////////////////////////////////////////
MSDialogP		OpenOrFindDialog ()
{
	MSDialogP	dbP	= mdlDialog_find (DIALOG_FileCollector, nullptr);
    if  (nullptr == dbP)
    {
		enum ResourceSearchOptions {	SearchAllOpenRscFiles	= 0, };
		dbP = mdlDialog_open (SearchAllOpenRscFiles, DIALOG_FileCollector);
	}
	return dbP;
}
/*----------------------------------------------------------------------+
|
|	fileCollector_test
|
|	Hidden command for arbitrary testing
|
|	Implements:				CMD_HARVEST_TEST
|
+----------------------------------------------------------------------*/
DLLEXPORT  void fileCollector_test
(
WCharCP				unparsed
)
{
	TraceFlags (gData);
}
/*----------------------------------------------------------------------+
|
|	fileCollector_exit
|
|	Exit file collector
|
|	Implements:				CMD_HARVEST_EXIT
|
+----------------------------------------------------------------------*/
DLLEXPORT  void 	fileCollector_exit
(
WCharCP				unparsed
)
{
	const bool		LocalCommand	= false;
	mdlDialog_cmdNumberQueue (LocalCommand, CMD_MDL_UNLOAD, mdlSystem_getCurrTaskID (), MSInputQueuePos::INPUTQ_EOQ);
}
/*----------------------------------------------------------------------+
|
|	SyncCheckBox
|
|	Set check box (toggle button) that sets the nominated flag
|
+----------------------------------------------------------------------*/
void				SyncCheckBox	(DialogItemIDs	checkBoxId)
{
	MSDialogP	dbP			= OpenOrFindDialog ();
	mdlDialog_itemSynchByTypeAndId (dbP, RTYPE_ToggleButton, checkBoxId);
}
/*----------------------------------------------------------------------+
|
|	fileCollector_setRecurse
|
|	Set flag for sub-folders
|
|	Implements:				CMD_HARVEST_RECURSE
|
+----------------------------------------------------------------------*/
DLLEXPORT  void 	fileCollector_setRecurse
(
WCharCP				unparsed
)
{
	switch (mdlCommandNumber)
	{
		case CMD_HARVEST_SET_SUBFOLDERS_ON:
			gData.flags.subFolders	= 1;
		break;
		case CMD_HARVEST_SET_SUBFOLDERS_OFF:
			gData.flags.subFolders	= 0;
		break;
		case CMD_HARVEST_SET_SUBFOLDERS_TOGGLE:
			gData.flags.subFolders	= !gData.flags.subFolders;
		break;
	}
	SyncCheckBox (CHECKBOX_SubFolders);
	TraceFlags (gData);
}
/*----------------------------------------------------------------------+
|
|	fileCollector_setCAD
|
|	Set flag for sub-folders
|
|	Implements:				CMD_HARVEST_CAD
|
+----------------------------------------------------------------------*/
DLLEXPORT  void 	fileCollector_setCAD
(
WCharCP				unparsed
)
{
	switch (mdlCommandNumber)
	{
		case CMD_HARVEST_SET_CAD_ON:
			gData.flags.cadFiles	= 1;
		break;
		case CMD_HARVEST_SET_CAD_OFF:
			gData.flags.cadFiles	= 0;
		break;
		case CMD_HARVEST_SET_CAD_TOGGLE:
			gData.flags.cadFiles	= !gData.flags.cadFiles;
		break;
	}
	SyncCheckBox (CHECKBOX_CAD_Files);
	TraceFlags (gData);
}
/*----------------------------------------------------------------------+
|
|	fileCollector_setWritable
|
|	Set flag for sub-folders
|
|	Implements:				CMD_HARVEST_WRITABLE
|
+----------------------------------------------------------------------*/
DLLEXPORT  void 	fileCollector_setWritable
(
WCharCP				unparsed
)
{
	switch (mdlCommandNumber)
	{
		case CMD_HARVEST_SET_WRITABLE_ON:
			gData.flags.writableFiles	= 1;
		break;
		case CMD_HARVEST_SET_WRITABLE_OFF:
			gData.flags.writableFiles	= 0;
		break;
		case CMD_HARVEST_SET_WRITABLE_TOGGLE:
			gData.flags.writableFiles	= !gData.flags.writableFiles;
		break;
	}
	SyncCheckBox (CHECKBOX_WritableFiles);
	TraceFlags (gData);
}
/*----------------------------------------------------------------------+
|
|	fileCollector_harvest
|
|	Harvest files from root folder
|
|	Implements:				CMD_HARVEST_FILES
|
+----------------------------------------------------------------------*/
DLLEXPORT  void 	fileCollector_harvest
(
WCharCP				root	//	=>	Possibly nullptr
)
{
	if (root && wcslen (root))
		wcscpy (gData.initialFolder, root);

	CMessageTracer	msg	(L"fileCollector_harvest "); msg.Quote (gData.initialFolder);
	//msg.Output ();

	//TraceFlags (gData);

	FileCollector		fileCollector;
	const size_t		nFiles	= fileCollector.Harvest (gData.initialFolder, gData.flags.subFolders, gData.flags.cadFiles, gData.flags.writableFiles);
	Bentley::WString	s;
	mdlOutput_rscsPrintf (s, 0, MESSAGELIST_Messages, MESSAGE_FileCount, nFiles);
	msg	= s;
	msg.Output (DgnPlatform::OutputMessagePriority::Info);

	//fileCollector.Trace ();

	MSDialogP			dbP			= OpenOrFindDialog ();
	DialogItemP			pListBox	= GetFileListBox (dbP);
	CListModel			listModel;
	listModel.AssignFromListBox (pListBox->rawItemP);
	listModel.ClearRows ();

	PathCollection const&	paths		= fileCollector.Paths ();

	for (PathConstIterator path = paths.begin (); path != paths.end (); ++path)
	{
		if (!listModel.AddRow (*path))
			break;
	}

	mdlDialog_listBoxNRowsChanged (pListBox->rawItemP);
    return;
}
//////////////////////////////////////////////////////////////////////
void 				listBoxItemHook
(
DialogItemMessage*	dimP
)
{
	RawItemHdr*		listBox		= dimP->dialogItemP->rawItemP;

    dimP->msgUnderstood= TRUE;

    switch  (dimP->messageType)
  	{
	    case DITEM_MESSAGE_CREATE:
   		{
			CListModel	listModel (listBox, CListModel::NotOwner);
			listModel.AssignColumnIDsFromListBox (listBox);
			listModel.AssignToListBox (listBox, false);
			break;
		}
	    case DITEM_MESSAGE_DESTROY:
		{
			CListModel	listModel;
			listModel.TakeOwnership (listBox);
	    	break;
 		}
	    case DITEM_MESSAGE_BUTTON:
		{
			int	         	row			= -1, col	= -1;
			int          	status;
			if  (BUTTONTRANS_UP == dimP->u.button.buttonTrans)
			{
				if (SUCCESS==(status = mdlDialog_listBoxLastCellClicked (&row, &col, listBox)))
				{
					//mdlDialog_listBoxEditCell (listBox, row, col);
				}
			}
			break;
		}
		case DITEM_MESSAGE_STATECHANGED:
		{
			if  (dimP->u.stateChanged.reallyChanged)
			{
			}
			break;
		}
		default:
			dimP->msgUnderstood = FALSE;
			break;
 	}
    return;
}
/////////////////////////////////////////////////////////////////////////////
void	InitialiseGlobalData	(GlobalData&	data)
{
    gData.colorNumber 			= 0;
	*gData.initialFolder		= 0;
	gData.flags.subFolders		= 1;
	gData.flags.cadFiles		= 0;
	gData.flags.writableFiles	= 0;
}
/////////////////////////////////////////////////////////////////////////////
///	<summary> 	OnReload Reload event handler.</summary>
void	OnReload			(int   argc, WCharCP	argv[])
{
	if (2 < argc)
	{
		//	User may supply an initial folder on the command line
		wcscpy_s (gData.initialFolder, MAXQUOTEDFILELENGTH, argv [2]);
	}
	else
	{
		//	Substitute your favourite initial folder here
		//wcscpy_s (gData.initialFolder, MAXQUOTEDFILELENGTH, L"L:\\FlexiTable\\dgn\\Hawes");
		MicroStation::GetCfgVar (gData.initialFolder, MAXQUOTEDFILELENGTH, L"MS_DEF");
	}
}
/////////////////////////////////////////////////////////////////////////////
///	<summary> 	OnUnload Unload event handler.</summary>
bool		OnUnload			(UnloadProgramReason   unloadType)
{
	return SUCCESS;
}
//////////////////////////////////////////////////////////////////////
///	<param name='argc' > The number of command line parameters sent to the application.<param>
//	<param name=argv[]' >     The array of strings sent to the application on the command line.<param>
//	extern "C" DLLEXPORT void MdlMain (int argc, WCharCP argv[]);
extern "C" DLLEXPORT  void MdlMain
(
int         argc,
WCharCP     argv[]
)
{
	RscFileHandle   rfHandle = 0;
    mdlResource_openFile (&rfHandle, nullptr, RSC_READONLY);

	InitialiseGlobalData (gData);

    DialogHookInfo uHooks[]=
    {
    	{HOOKITEM_ListBox,  (PFDialogHook) listBoxItemHook},
    };

    mdlDialog_hookPublish (sizeof (uHooks)/sizeof (uHooks[0]), uHooks);

    SymbolSet*		setP = mdlCExpression_initializeSet (VISIBILITY_DIALOG_BOX, 0, TRUE);
    mdlDialog_publishComplexVariable (setP, "globaldata", "gData", &gData);

    // Map key-in to function
    static MdlCommandNumber cmdNumbers[] =
    {
		{(CmdHandler)fileCollector_harvest,  	CMD_HARVEST_FILES 					},
        {(CmdHandler)fileCollector_setCAD,  	CMD_HARVEST_SET_CAD_ON 				},
        {(CmdHandler)fileCollector_setCAD,  	CMD_HARVEST_SET_CAD_OFF				},
        {(CmdHandler)fileCollector_setCAD,  	CMD_HARVEST_SET_CAD_TOGGLE 			},
        {(CmdHandler)fileCollector_setRecurse,  CMD_HARVEST_SET_SUBFOLDERS_ON 		},
        {(CmdHandler)fileCollector_setRecurse,  CMD_HARVEST_SET_SUBFOLDERS_OFF		},
        {(CmdHandler)fileCollector_setRecurse,  CMD_HARVEST_SET_SUBFOLDERS_TOGGLE 	},
        {(CmdHandler)fileCollector_setWritable,	CMD_HARVEST_SET_WRITABLE_ON 		},
        {(CmdHandler)fileCollector_setWritable,	CMD_HARVEST_SET_WRITABLE_OFF		},
        {(CmdHandler)fileCollector_setWritable,	CMD_HARVEST_SET_WRITABLE_TOGGLE 	},
        {(CmdHandler)fileCollector_test,  		CMD_HARVEST_TEST 					},
        {(CmdHandler)fileCollector_exit,  		CMD_HARVEST_EXIT 					},
        0,
    };

    mdlSystem_registerCommandNumbers (cmdNumbers);

    mdlParse_loadCommandTable (nullptr);

    //	The MDL way
    //VersionNumber   v   = { 16, 11, 29, 0 };
    //MicroStation::SetVersionNumber (v);
    //	The Windows way
	MicroStation::SetVersion (TXT_DllName);

	SystemCallback::SetReloadProgramFunction (OnReload);
	SystemCallback::SetUnloadProgramFunction (OnUnload);

	OnReload (argc, argv);

    OpenOrFindDialog ();
}
