#if !defined(FILE_FUNCTORS_H_INCLUDED_)
#define FILE_FUNCTORS_H_INCLUDED_

#pragma once

#include	"MicroStation.h"
#include	<UndefineMdlMacrosThatInterfereWithBoost.h>
#include 	<boost/filesystem/operations.hpp>
#include	<string>
#include	<tstring_typedef.h>
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates files that are not DGN or DWG format.</summary>
///	<returns>	True if file cannot be opened by MicroStation.</returns>
struct WrongVersionDgnFile : public std::unary_function<boost::filesystem::path const&, bool>
{
	UInt32		flags_;		//	Combination of FileCollector::CollectFileFlags

	WrongVersionDgnFile	(UInt32 flags) : flags_	(flags)	{};
	bool	operator()(boost::filesystem::path const& fileEntry);
};
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates files that are in a specified format.</summary>
///	<returns>	True if file can be opened by MicroStation.</returns>
struct CadFile : public std::unary_function<boost::filesystem::path const&, bool>
{
	UInt32		flags_;		//	Combination of FileCollector::CollectFileFlags

	CadFile	(UInt32 flags) : flags_	(flags)	{};
	bool	operator()(boost::filesystem::path const& fileEntry);
};
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates objects that are shortcuts (links, .lnk).
///	<remarks>boost::filesystem provides this ability, which was not available
///	with the previous API.<para>This functor is consquently redundant.</para></remarks>
///	</summary>
struct IsLink : public std::unary_function<boost::filesystem::path const&, bool>
{
	bool	operator()(boost::filesystem::path const& fileEntry);
};
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates files that are not DGN or DWG format.</summary>
///	<returns>	True if file cannot be opened by MicroStation.</returns>
struct InvalidDesignFile : public std::unary_function<boost::filesystem::path const&, bool>
{
	bool	operator()(boost::filesystem::path const& fileEntry);
};
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates files that are read-only.</summary>
///	<returns>	True if file cannot be opened for write access.</returns>
struct ReadOnlyFile : public std::unary_function<boost::filesystem::path const&, bool>
{
	bool	operator()(boost::filesystem::path  const& fileEntry);
};
//////////////////////////////////////////////////////////////////////
///	<summary>Functor predicates files that are available for write.
///	<remarks>Obsolete with boost::filesystem.</remarks>
///	</summary>
///	<returns>	True if file cannot be opened for write access.</returns>
struct WriteableFile : public std::unary_function<boost::filesystem::path const&, bool>
{
	bool	operator()(boost::filesystem::path const& fileEntry);
};
#endif // !defined(FILE_FUNCTORS_H_INCLUDED_)
