#include    <Mstn\MdlApi\MdlApi.h>
#include	"List_Model.h"
#include 	"BoostFileSystem.h"
#include 	"FileFilterFunctors.h"
#include 	"MessageTracer.h"
///////////////////////////////////////////////////////////////////////
using namespace boost::filesystem;
//////////////////////////////////////////////////////////////////////
CListModel::CListModel	(/* constructor 1 */)
: pListModel_ (nullptr), ownsList_ (NotOwner)
{
	//CMessageTracer msg	("CListModel constructor 1 ");
	//msg.Output ();
}
//////////////////////////////////////////////////////////////////////
CListModel::CListModel	(RawItemHdr*	pListBox, ListOwnership owner)
{
	//	If we've been told to take ownership then we are the owner and will
	//	destroy the ListModel in our destructor
	ownsList_	= owner;

    pListModel_ = mdlListModel_create (mdlDialog_listBoxGetNColumns (pListBox));
	//CMessageTracer msg	("CListModel constructor 3 owner "); msg.Quote (ToString (owner));
	//msg.Append (" N columns="); msg.Append (mdlListModel_getColumnCount (pListModel_));
	//msg.Output ();
}
//////////////////////////////////////////////////////////////////////
CListModel::CListModel	(int nCols, ListOwnership owner)
{
	//CMessageTracer msg	("CListModel constructor 2 N columns="); msg.Append (nCols);
	//msg.Output ();
    pListModel_ = mdlListModel_create (nCols);
}
//////////////////////////////////////////////////////////////////////
void	CListModel::TakeOwnership	(RawItemHdr*	pListBox)
{
	ownsList_	= Owner;
	pListModel_ = mdlDialog_listBoxGetListModelP (pListBox);
}
//////////////////////////////////////////////////////////////////////
CListModel::~CListModel()
{
	Destroy ();
}
//////////////////////////////////////////////////////////////////////
void 	CListModel::Destroy ()
{
	if (pListModel_ && Owner == ownsList_)
	{
		mdlListModel_destroy (pListModel_, DestroyRows);
	}
}
//////////////////////////////////////////////////////////////////////
//	static
std::wstring		CListModel::DescribeColumnId (ColumnIds		id)
{
	switch (id)
	{
    case FileTitle:			return std::wstring (L"FileTitle");
    case FileName:			return std::wstring (L"FileName");
    case FilePath:			return std::wstring (L"FilePath");
	case FileExtension:		return std::wstring (L"FileExtension");
	case TypeDescription:	return std::wstring (L"TypeDescription");
    case Writeable:			return std::wstring (L"Writeable");
    case IsCadFile:			return std::wstring (L"IsCadFile");
    case Directory:			return std::wstring (L"Directory");
	}
	return std::wstring (L"Unknown Column ID");
}
//////////////////////////////////////////////////////////////////////
/*
std::wstring	CListModel::Describe		(ListOwnership					ownership)	const
{
	std::wstring	s (L"Invalid Ownership");
	switch (ownership)
	{
		case Owner: 		s = L"Owner"; 					break;
		case TakeOwnership:	s = L"TakeOwnership";	break;
		case NotOwner:		s = L"NotOwner";				break;
	}
	return s;
}
*/
//////////////////////////////////////////////////////////////////////
void	CListModel::ClearRows ()
{
	 mdlListModel_empty (pListModel_, DestroyRows);
}
//////////////////////////////////////////////////////////////////////
bool	CListModel::AssignColumn (WChar const* colName, ColumnIds id, RawItemHdr*	pListBox, int col)
{
	const bool		NoRedraw	= false;
	//CMessageTracer msg	("CListModel::AssignColumn "); msg.Append (col); msg.Space (); msg.Quote (colName);
	//msg.Append (" ID "); msg.Quote (DescribeColumnId (id));
	//	msg.Output ();
	WChar			heading		[42];
	WString			caption;
	mdlDialog_listBoxGetColInfo (nullptr, nullptr, nullptr, heading, pListBox, col);
	const bool		matched		(0 == wcsicmp (colName, heading));
	if (matched)
	{
		//msg = "Matched heading "; msg.Quote (heading); msg.Append (" get caption ID "); msg.Quote (DescribeColumnId (id));
		//msg.Output ();
		ListColumnP	pCol	= mdlListModel_getColumnAtIndex (pListModel_, col);
		mdlListColumn_setId (pCol, id);
		mdlResource_loadWString (caption, 0, MESSAGELIST_Captions, id);
		mdlListColumn_setName (pCol, caption.c_str ());
		mdlDialog_listBoxSetColInfo (nullptr, nullptr, nullptr, caption.c_str (), pListBox, col, NoRedraw);
	}
	return matched;
}
//////////////////////////////////////////////////////////////////////
void	CListModel::AssignColumnIDsFromListBox	(RawItemHdr*	pListBox)
{
	const int	nColumns	= mdlDialog_listBoxGetNColumns (pListBox);
	//CMessageTracer msg	("AssignColumnIDsFromListBox N columns="); msg.Append (nColumns);
	//msg.Output ();
	for (int col = 0; col != nColumns; ++col)
	{
		 if      (AssignColumn (L"FileTitle",		FileTitle,		pListBox, col)) {}
		 else if (AssignColumn (L"FileName", 		FileName,		pListBox, col)) {}
		 else if (AssignColumn (L"FilePath", 		FilePath,		pListBox, col)) {}
		 else if (AssignColumn (L"FileExtension", 	FileExtension,	pListBox, col)) {}
		 else if (AssignColumn (L"TypeDescription",	TypeDescription, pListBox, col)) {}
		 else if (AssignColumn (L"Writeable", 		Writeable, 		pListBox, col)) {}
		 else if (AssignColumn (L"IsCadFile", 		IsCadFile, 		pListBox, col)) {}
		 else if (AssignColumn (L"Directory", 		Directory, 		pListBox, col)) {}
	}
}
//////////////////////////////////////////////////////////////////////
int		CListModel::ColumnCount					()	const
{

	if (nullptr != pListModel_)
		return mdlListModel_getColumnCount (pListModel_);
	//else
	return 0;
}
//////////////////////////////////////////////////////////////////////
void	CListModel::AssignFromListBox	(RawItemHdr*	pListBox, ListOwnership owner)
{
	pListModel_ = mdlDialog_listBoxGetListModelP (pListBox);
	//CMessageTracer	msg ("CListModel::AssignFromListBox pListModel_ is ");
	//	msg.Append (nullptr == pListModel_? "nullptr": "valid");
	//msg.Output ();
	//	If we've been assigned the pointer from a ListBox DESTROY message,
	//	it's our job to deallocate the ListModel when our instance falls out of scope
	ownsList_	= owner;
}
//////////////////////////////////////////////////////////////////////
void	CListModel::AssignToListBox	(RawItemHdr*	pListBox, bool refresh)	const
{
	//CMessageTracer	msg ("CListModel::AssignToListBox ListBox N columns=");
	//	msg.Append (mdlDialog_listBoxGetNColumns (pListBox));
	//	msg.Append (" ListModel N columns="); msg.Append (ColumnCount ());
	//msg.Output ();
	mdlDialog_listBoxSetListModelP (pListBox, pListModel_, mdlListModel_getColumnCount (pListModel_));
	if (refresh)
	{
		mdlDialog_listBoxNRowsChanged (pListBox);
	}
}
//////////////////////////////////////////////////////////////////////
bool	CListModel::AddRow	(WCharCP	word)
{
	ListRowP	pRow = mdlListRow_create (pListModel_);
	for (int colIndex = 0; 0 != pRow && colIndex != mdlListModel_getColumnCount (pListModel_); ++colIndex )
	{
		ListCellP	pCell = mdlListRow_getCellAtIndex (pRow, colIndex);
		mdlListCell_setStringValue (pCell, word, TRUE);
	}
	const int	status	= mdlListModel_addRow (pListModel_, pRow);
	return (0 != pRow && MDLERR_ADDRNOTVALID != status && MDLERR_INSFMEMORY != status);
}
//////////////////////////////////////////////////////////////////////
bool	CListModel::AssignFileData	(boost::filesystem::path const&	p,
									ListCellP						pCell,
									ColumnIds 						id,
									int								index)
{
	//CMessageTracer	msg	("CListModel::AssignFileData to cell ");
	//msg.Append (" columns ID "); msg.Append (id); msg.Append (" index "); msg.Append (index);
	//msg.Output ();
	bool		rc		(false);
	file_status fs 		= status (p);
	perms		pm		= fs.permissions ();

	if (fs.type () == regular_file)
	{
		switch (id)
		{
			case FileTitle:
			{	//	Create a temporary path object
				path	title	(p);
				FileCollector::RemoveExtension (title);
				//	Display the file title without extension
				mdlListCell_setDisplayText (pCell, title.filename ().wstring ().c_str ());
				rc	= true;
				break;
			}
			case FileName:
				 mdlListCell_setDisplayText (pCell, p.filename ().wstring ().c_str ());
				 rc	= true;
			break;
			case FilePath:
				 rc	= true;
				 mdlListCell_setDisplayText (pCell, p.parent_path ().wstring ().c_str ());
			break;
			case FileExtension:
				 mdlListCell_setDisplayText (pCell, p.extension ().wstring ().c_str ());
				 rc	= true;
			break;
			case TypeDescription:
			{
				mdlListCell_setDisplayText (pCell, FileCollector::DescribeFileType (p).c_str ());
				break;
			}
			case Writeable:
				mdlListCell_setDisplayText (pCell, (pm & owner_write)? L"Writeable": L"Read Only");
				rc	= true;
			break;
			case IsCadFile:
			{
				CadFile	test (FileCollector::CadFiles);
				mdlListCell_setDisplayText (pCell, test (p)? L"CAD File": L"Not CAD File");
				rc	= true;
				break;
			}
		}
	}
	else if (fs.type () == directory_file)
	{
		switch (id)
		{
			case Directory:
			break;
		}
	}
	return rc;
}
//////////////////////////////////////////////////////////////////////
bool	CListModel::AddRow	(boost::filesystem::path const&	p)
{
	const int			nColumns	= mdlListModel_getColumnCount  (pListModel_);
	//CMessageTracer	msg	("CListModel::AddRow "); msg.Quote (p.wstring ().c_str ());
	//msg.Append (" N columns="); msg.Append (nColumns);
	//msg.Output ();
	ListRowP			pRow 		= mdlListRow_create (pListModel_);
	for (int col = 0; col != nColumns; ++col)
	{
		ListColumnP		pCol	= mdlListModel_getColumnAtIndex (pListModel_, col);
		const ColumnIds	colID	= static_cast<ColumnIds>(mdlListColumn_getId (pCol));
		AssignFileData (p, mdlListRow_getCellById (pRow, colID), colID, col);
	}
	const int	status	= mdlListModel_addRow (pListModel_, pRow);
	return (0 != pRow && MDLERR_ADDRNOTVALID != status && MDLERR_INSFMEMORY != status);
}
//////////////////////////////////////////////////////////////////////
