#if !defined(LIST_MODEL_H_INCLUDED_)
#define LIST_MODEL_H_INCLUDED_

#pragma once

//	This header is called List_Model.h to distinguish it from the MDL header
#include    <Mstn\MdlApi\dlogbox.r.h>
#include	"FileCollector.h"
#include	<UndefineMdlMacrosThatInterfereWithBoost.h>
#include 	<SelfDocumentingEnum.h>
#include 	<boost/filesystem/operations.hpp>

class CListModel
{
public:

	/*
	enum	ListOwnership
	{
		Owner,
		NotOwner,
	};
	*/

	DEFINE_ENUM_WITH_STRING_CONVERSIONS(ListOwnership, (Owner)(NotOwner))

private:
	std::wstring	Describe					(ListOwnership					ownership)	const;
	enum RowControl	{  DestroyRows	= TRUE,	};

	ListModelP		pListModel_;
	ListOwnership	ownsList_;

	void			Destroy						();
	bool			AssignColumn 				(WChar const* 					colName,
												ColumnIds 						id,
												RawItemHdr*						pListBox,
												int 							col);
	bool			AssignFileData				(boost::filesystem::path const&	p,
												ListCellP						pCell,
												ColumnIds 						id,
												int								index);
public:
	static std::wstring	DescribeColumnId 		(ColumnIds						id);
	///	<summary>Get column names from ListBox and set Column IDs.
	///	<remarks>This enables us to define which columns are displayed from the ListBox resource.</remarks>
	///	</summary>
	void			AssignColumnIDsFromListBox	(RawItemHdr*					pListBox);
	///	<summary>	Return the number of columns in this ListModel.</summary>
	int				ColumnCount					()	const;
	///	<summary>	Delete data from this ListModel.</summary>
	void			ClearRows ();
	///	<summary>	Assign our ListModel from a ListBox dialog item.</summary>
	void			AssignFromListBox	(RawItemHdr*	pListBox, ListOwnership owner = NotOwner);
	///	<summary>	Assign our ListModel to a ListBox dialog item.</summary>
	void			AssignToListBox	(RawItemHdr*		pItem, bool refresh = true)	const;
	///	<summary>	Return a pointer to this object's ListModel.</summary>
	ListModelP		GetPointer	()	{	return pListModel_;	}
	///	<summary>	Add a row to our ListModel.
	///	<remarks>	Set the display in each column to the specified text.
	///	<para>		Not used in this application.</para></remarks></summary>
	bool			AddRow	(WChar const*	word);
	///	<summary>	Add a ListRow that stores information about a file path.</summary>
	bool			AddRow	(boost::filesystem::path const&	p);
	///	<summary>	Take ownership of an existing ListModel from a ListBox.</summary>
	void			TakeOwnership	(RawItemHdr*	pListBox);
	///	<summary>	Construction</summary>
	CListModel	(/* constructor 1 */);
	CListModel	(/* constructor 2 */int nCols, 	ListOwnership owner	= NotOwner);
	///	<summary>	Deduce column count and IDs from ListBox resource data.</summary>
	CListModel	(/* constructor 3 */RawItemHdr*	pListBox, ListOwnership owner	= NotOwner);
	~CListModel ();
};

#endif // !defined(LIST_MODEL_H_INCLUDED_)
