/*----------------------------------------------------------------------+
|																		|
| MessageTracer Copyright 2016, LA Solutions Ltd						|
| All rights reserved.													|
|																		|
| Limited permission is hereby granted to reproduce and modify this		|
| copyrighted material provided that this notice is retained			|
| in its entirety in any such reproduction or modification.				|
|																		|
+----------------------------------------------------------------------*/
/*----------------------------------------------------------------------+
|                                                                       |
|   MessageTracer - MessageTracer source code. 							|
|                                                                       |
+----------------------------------------------------------------------*/

#include 	"MessageTracer.h"
#include 	"ModulVer.h"
#include    <Mstn\MdlApi\MdlApi.h>
///////////////////////////////////////////////////////////////////////
//	Implementation
///////////////////////////////////////////////////////////////////////
//	static
std::wstring	CMessageTracer::Convert			(char const*		s)
{
	const size_t	len		= strlen (s);
	WChar*			temp	= new WChar [1 + len]; *temp	= 0;
	mbstowcs (temp, s, len);
	 *(len + temp)	= 0;
	std::wstring	ws (temp);
	delete [] temp;
	return ws;
}
///////////////////////////////////////////////////////////////////////
//	static
std::wstring	CMessageTracer::Convert			(std::string const&	s)
{
	return Convert (s.c_str ());
}
///////////////////////////////////////////////////////////////////////
//	Output to MicroStation's message center.  If verbose_ is empty,
//	send terse_ to the message center list and detail window;
//	if verbose_ is not empty, send it to the detail window
void			CMessageTracer::Output	(DgnPlatform::OutputMessagePriority				priority)	const
{
	mdlOutput_messageCenter (priority, terse_.c_str (), verbose_.empty ()? terse_.c_str (): verbose_.c_str (), DgnPlatform::OutputMessageAlert::None);
}
///////////////////////////////////////////////////////////////////////
//	Output to MicroStation's message center.  If verbose_ is empty,
//	send terse_ to the message center list and detail window;
//	if verbose_ is not empty, send it to the detail window
void			CMessageTracer::Warning	(DgnPlatform::OutputMessagePriority				priority,
										bool				pop)	const
{
	mdlOutput_messageCenter (priority, terse_.c_str (), verbose_.empty ()? terse_.c_str (): verbose_.c_str (),
		pop? DgnPlatform::OutputMessageAlert::Dialog: DgnPlatform::OutputMessageAlert::None);
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const char*			msg)
{
	terse_		= Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const std::string&		msg)
{
	terse_		= Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const WChar*			msg)
{
	terse_		= msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const std::wstring&	msg)
{
	terse_		= msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&		CMessageTracer::operator=	(const Bentley::WString&	msg)
{
	terse_		= msg.c_str ();
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const char*			msg)
{
	terse_	+=	Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const std::string&		msg)
{
	terse_	+=	Convert (msg);
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const WChar*			msg)
{
	terse_	+=	msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const std::wstring&	msg)
{
	terse_	+=	msg;
	return *this;
}
///////////////////////////////////////////////////////////////////////
CMessageTracer&	CMessageTracer::operator +=	(const Bentley::WString&	msg)
{
	terse_	+=	msg.c_str ();
	return *this;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Space	(int count)
{
	const std::wstring	Space (L" ");
	for (int i = 0; i != count; ++i)
	{
		terse_	+= Space;
	}
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const char*		msg)
{
	const std::wstring	Quote (L"'");
	terse_		+= Quote;
	terse_		+= Convert (msg);
	terse_		+= Quote;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const std::string&	msg)
{
	Quote (msg.c_str ());
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const WChar*			msg)
{
	const std::wstring	Quote (L"'");
	terse_		+= Quote;
	terse_		+= msg;
	terse_		+= Quote;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Quote	(const std::wstring&	msg)
{
	Quote (msg.c_str ());
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const char*			msg)
{
	terse_		+= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const std::string&		msg)
{
	terse_		+= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const wchar_t*			msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(const std::wstring&	msg)
{
	terse_		+= msg;
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(bool					state)
{
	terse_		+= state? std::wstring (L"true"): std::wstring (L"false");
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(int					number)
{
	WChar			temp	[128];
	swprintf (temp, L"%d", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(long				number)
{
	WChar			temp	[128];
	swprintf (temp, L"%ld", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(UInt32				number)
{
	WChar			temp	[128];
	swprintf (temp, L"%ud", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(size_t				number)
{
	WChar			temp	[128];
	swprintf (temp, L"%ud", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(Bentley::DgnPlatform::ElementId const&	number)
{
	WChar			temp	[128];
	swprintf (temp, L"%I64d", number);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Degrees	(double				radians)
{
	WChar			temp	[128];
	swprintf (temp, L" %.0lf", radians * fc_180overpi);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Radians	(double				radians)
{
	WChar			temp	[128];
	swprintf (temp, L" %.4lf", radians);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Euler	(RotMatrix const&	r,
										int					precision)
{
	WChar			temp	[128];
	double			x, y, z, s;
	if (mdlRMatrix_isXRotationYRotationZRotationScale (&r, &x, &y, &z, &s))
		swprintf (temp, L" x='%.4lf' y='%.4lf' z='%.4lf' ", x, y, z);
	else
		swprintf (temp, L" matrix has non-uniform scale");
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(double				d,
										DgnModelRefP		modelRef,
										int					precision)
{
	const double	UORs	= 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	Format	(L"%%.%dlf");
	swprintf (format, Format, precision);
	swprintf (temp, format, d * UORs);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(RotMatrix const&	r,
										int					precision)
{
	WChar			format	[128];
	WChar			temp	[128];
	const WChar*	FormatRow	(L"\n| %%.%dlf, %%.%dlf, %%.%dlf |");
	swprintf (format, FormatRow, precision, precision, precision);
	swprintf (temp, format, r.form3d [0][0], r.form3d [0][1], r.form3d [0][2]);
	terse_		+= std::wstring (temp);
	swprintf (temp, format, r.form3d [1][0], r.form3d [1][1], r.form3d [1][2]);
	terse_		+= std::wstring (temp);
	swprintf (temp, format, r.form3d [2][0], r.form3d [2][1], r.form3d [2][2]);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Vector	(DVec3d const&		v,
										int					precision)
{
	WChar			format	[128];
	WChar			temp	[128];
	WChar const*	Format3D	(L"%%.%dlf,%%.%dlf,%%.%dlf");
	swprintf (format, Format3D, precision, precision, precision);
	swprintf (temp, format, v.x, v.y, v.z);
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::IdentifyElm	(MSElementDescrCP	pElm)
{
	WChar			temp	[128];
	swprintf (temp, L" ID %I64d type %d", mdlElement_getID (&pElm->el), mdlElement_getType (&pElm->el));
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Append	(DPoint3d const&	pt,
										DgnModelRefP		modelRef,
										int					precision)
{
	const double	UORs	= 1.0 / mdlModelRef_getUorPerMaster (modelRef);
	WChar			temp	[128];
	WChar			format	[128];
	if (mdlModelRef_is3D (modelRef))
	{
		const WChar*		Format3D	(L"x='%%.%dlf' y='%%.%dlf' z='%%.%dlf'");
		swprintf (format, Format3D, precision, precision, precision);
		swprintf (temp, format, pt.x * UORs, pt.y * UORs, pt.z * UORs);
	}
	else
	{
		const WChar*		Format2D	(L"x='%%.%dlf' y='%%.%dlf'");
		swprintf (format, Format2D, precision, precision);
		swprintf (temp, format, pt.x * UORs, pt.y * UORs);
	}
	terse_		+= std::wstring (temp);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Message	(const char*		msg)
{
	terse_		= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Message	(const std::string&	msg)
{
	terse_		= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(const char*		msg)
{
	verbose_	= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
void			CMessageTracer::Verbose	(const std::string&	msg)
{
	verbose_	= Convert (msg);
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const char*			msg)
:	terse_								(Convert (msg)),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const std::string&		msg)
:	terse_								(Convert (msg)),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const WChar*			msg)
:	terse_								(msg),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
//	Construction
CMessageTracer::CMessageTracer			(const std::wstring&	msg)
:	terse_								(msg),
	verbose_							(L"")
{
}
///////////////////////////////////////////////////////////////////////
