#include    <Mstn/MdlApi/MdlApi.h>
#include	"MicroStation.h"
#include	"ModulVer.h"
#include	<string>
////////////////////////////////////////////////////////////////////////////////////////////////////
MicroStation::BusyCursor::BusyCursor    ()
{
    mdlSystem_startBusyCursor ();
}
MicroStation::BusyCursor::~BusyCursor   ()
{
    mdlSystem_stopBusyCursor ();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void				MicroStation::GetCfgVar (WChar* buffer, size_t size, WChar const* cfgVar)
{
	const WString		value	(GetCfgVar (cfgVar));
	wcscpy_s (buffer, size, value.c_str ());
}
////////////////////////////////////////////////////////////////////////////////////////////////////
WString				MicroStation::GetCfgVar 		(WChar const* cfgVar)
{
	WString    value;
	if (SUCCESS != mdlSystem_getCfgVar(&value, cfgVar))
	{
		value	= L"Invalid CfgVar '";
		value	+= WString (cfgVar); value	+= WString (L"'");
	}
	return value;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
DgnPlatform::DgnFileFormatType   MicroStation::FileFormat 	(WChar const*        	filePath,
															bool*                   is3D /* = nullptr */)
{
	DgnPlatform::DgnFileFormatType	dgnFormat           {DgnPlatform::DgnFileFormatType::Invalid};   //  DGNFILE_FORMAT_XXX
	int                 			major               {0};
	int                 			minor               {0};
	bool             				defaultModelIs3D    {false};
	int                 			imageSize           {0};
	DgnPlatform::DgnFileFormatType	format              {DgnPlatform::DgnFileFormatType::Invalid};
	if (mdlSystem_openFileThumbnail (&format, &major, &minor, &defaultModelIs3D, filePath, nullptr, &imageSize))
	{
		dgnFormat   = format;
	}
	if (is3D)
	{
		*is3D   	= defaultModelIs3D;
	}
	return dgnFormat;
}
//////////////////////////////////////////////////////////////////////
bool    MicroStation::CanOpen                   (WChar const*                 filePath)
{
    bool    						valid              	{false};
    DgnPlatform::DgnFileFormatType	format              {DgnPlatform::DgnFileFormatType::Invalid};
    int     						major               {0};
    int     						minor               {0};
    bool 							defaultModelIs3D  	{false};
    int     						imageSize           {0};

    if (mdlSystem_openFileThumbnail     (&format, &major, &minor, &defaultModelIs3D, filePath, nullptr, &imageSize))
    {
        valid   =   (DgnPlatform::DgnFileFormatType::V8  == format
                    ||
                    DgnPlatform::DgnFileFormatType::V7   == format
                    ||
                    DgnPlatform::DgnFileFormatType::DWG  == format
                    ||
                    DgnPlatform::DgnFileFormatType::DXF  == format);
    }
    return valid;
}
//////////////////////////////////////////////////////////////////////
//  Microsoft C++ compiler doesn't understand MDL's #pragma Version,
//  so we set the version programmatically
void    	MicroStation::SetVersionNumber        (VersionNumber const& v)
{
    mdlSystem_setMdlAppVersionNumber (mdlSystem_getCurrMdlDesc (), const_cast<VersionNumber*>(&v));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
//	Helper function for SetVersion
int			VersionPart (size_t& pos0, size_t& pos1, std::wstring const& s, WChar const* separator, WChar const* context)
{
	pos1    = s.find (separator, pos0);
	const int v	= wcstol (s.substr (pos0, pos1 - pos0).c_str (), nullptr, 10);
	pos0	= pos1 + 1;
	return v;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void        MicroStation::SetVersion (WChar const*   dllName)
{
    CModuleVersion	ver;
    if (ver.GetFileVersionInfo (dllName))
    {
        //  release.major.minor.subMinor
        VersionNumber   msVersion;
        //  The format string is in the same form as the VERSION_INFO
        //  resource: 1, 2, 3, 4 or 1.2.3.4 (i.e. separator may be period or comma.  Possibly Viz Studio version dependent)
        std::wstring 	s (ver.GetValue (L"FileVersion"));
        LPCWSTR     	Comma       { L"," };
        LPCWSTR     	Period   	{ L"." };
		LPCWSTR			separator	{ Comma };
        size_t      	pos0        { 0 };
        size_t      	pos1     	{ s.find (separator, pos0) };
		//	Determine what separator is used
        if (std::wstring::npos == pos1)
        {
			separator	= Period;
			pos1		= s.find (separator, pos0);
		}
		msVersion.release	= VersionPart (pos0, pos1, s, separator, L"msVersion.release");
		msVersion.major		= VersionPart (pos0, pos1, s, separator, L"msVersion.major");
		msVersion.minor		= VersionPart (pos0, pos1, s, separator, L"msVersion.minor");
		msVersion.subMinor	= VersionPart (pos0, pos1, s, separator, L"msVersion.subMinor");

        mdlSystem_setMdlAppVersionNumber (nullptr, &msVersion);
    }
    else
    {
		LPCWSTR		error {  L"SetVersion: GetFileVersionInfo failed" };
		mdlOutput_messageCenter (DgnPlatform::OutputMessagePriority::Error, error, error, DgnPlatform::OutputMessageAlert::Dialog);
	}
}
